package cn.kdan.cloud.pdf.office.payment.controller;

import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.service.WebhookService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/2/6
 * <p>
 * OrderController
 */
@RestController
@RequestMapping("/order")
@RequiredArgsConstructor
public class OrderController {

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final ProductApi productApi;

    private final WebhookService webhookService;

    private final AppStoreService appStoreService;

    /**
     * 创建支付订单
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/createOrder")
    public ResultMap<CreateOrderResultVO> createOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO){
        return ResultMap.success(ordersService.createOrder(createOrderDTO));
    }

    /**
     * 订单创建（applePay专用）
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/appStoreCreateOrder")
    public ResultMap<CreateOrderResultVO> appStoreCreateOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO){
        return ResultMap.success(ordersService.appStoreCreateOrder(createOrderDTO));
    }

    /**
     * 订单支付成功（applePay专用）
     * 回传appal pay id &签名证书
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return Void
     */
    @PostMapping("/appStoreOrderSucceed")
    public ResultMap<UserInfoVO> appStoreOrderSucceed(@Validated @RequestBody AppStoreOrderSucceedDTO appStoreOrderSucceed){
        return ResultMap.success(appStoreService.appStoreOrderSucceed(appStoreOrderSucceed));
    }

    /**
     * APP Store购买完成
     * @param dto 1
     * @return
     */
    @PostMapping("/appStoreBuyComplete")
    public ResultMap<Void> appStoreBuyComplete(@Validated @RequestBody AppStoreBuyCompleteDTO dto){
        appStoreService.appStoreBuyComplete(dto);
        return ResultMap.success();
    }

    /**
     * appStore 权益校验
     *
     * @param equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    @PostMapping("/appStoreEquityVerification")
    public ResultMap<UserInfoVO> appStoreEquityVerification(@RequestBody EquityVerificationDTO equityVerificationDTO){
        return ResultMap.success(appStoreService.appStoreEquityVerification(equityVerificationDTO));
    }
    /**
     * 手动创建虚拟订单
     *
     * @param createOrderManual createOrderManual
     * @return OrderUpdateVO
     */
    @PostMapping("/createOrderManual")
    public ResultMap<OrderUpdateVO> createOrderManual(@Validated @RequestBody CreateOrderManualDTO createOrderManual){
        return ResultMap.success(ordersService.createOrderManual(createOrderManual));
    }

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderById")
    public ResultMap<OrdersVO> getOrderById(@Validated @NotNull(message = "orderId不能为空")
                                            @RequestParam("orderId") String orderId){
        return ResultMap.success(ordersService.getOrderById(orderId));
    }

    /**
     * 查询订单详情（用于判断订阅是否已经支付完成）
     * @param orderId
     * @return OrdersVO
     */
    @GetMapping("/getOrderWithUpdate")
    public ResultMap<OrdersVO> getOrderWithUpdate(@Validated @NotNull(message = "orderId不能为空")
                                            @RequestParam("orderId") String orderId){
        return ResultMap.success(webhookService.getOrderWithUpdate(orderId));
    }

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    @GetMapping("/getOrderByTradeNo")
    public ResultMap<List<OrdersVO>> getOrderByTradeNo(@Validated @NotNull(message = "tradeNo不能为空")
                                                 @RequestParam("tradeNo") String tradeNo){
        return ResultMap.success(ordersService.getOrderByTradeNo(tradeNo));
    }

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     * @return Void
     */
    @PostMapping("/chargebackOrder")
    public ResultMap<Void> chargebackOrder(@Validated @RequestBody ChargebackOrderDTO chargebackOrderDTO){
        // 修改 订单状态
        ordersService.chargebackOrder(chargebackOrderDTO);
        // 修改 订阅状态
        subscriptionsService.chargebackSubscription(chargebackOrderDTO);
        return ResultMap.success();
    }

    /**
     * 取消续订
     *
     * @param cancelSubscription cancelSubscription
     * @return Void
     */
    @PostMapping("/cancelSubscription")
    public ResultMap<Void> cancelSubscription(@Validated @RequestBody CancelSubscriptionDTO cancelSubscription) {
        subscriptionsService.chargebackSubscription(cancelSubscription);
        return ResultMap.success();
    }

}
