package cn.kdan.cloud.pdf.office.payment.webhook;

import cn.kdan.cloud.pdf.office.api.payment.constant.PaddleAPIConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreWebhookService;
import cn.kdan.cloud.pdf.office.payment.service.PaddleWebhookService;
import cn.kdan.cloud.pdf.office.payment.utils.JwsUtil;
import cn.kdan.cloud.pdf.office.payment.webhook.appstore.NotificationDecoder;
import cn.kdan.cloud.pdf.office.payment.webhook.appstore.notification.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.http.*;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.client.RestTemplate;

import java.security.cert.CertificateException;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/6/20
 */
@Controller
@RequestMapping("/appstore")
@RequiredArgsConstructor
@Slf4j
public class AppStoreWebhookMonitor {

    private final JwsUtil jwsUtil;

    private final NotificationDecoder notificationDecoder = new NotificationDecoderImpl();

    private final RedissonClient redissonClient;

    private final AppStoreWebhookService appStoreWebhookService;

    @RequestMapping(value = "/webhook/subscription", method = RequestMethod.POST)
    public ResponseEntity<String> receiveSubscriptionCallback(@RequestBody ResponseBodyV2 responseBodyV2) {
        log.info("appstore 接收回调信息JWT值：{}", responseBodyV2);
        try {
            ResponseBodyV2DecodedPayload decodedPayload = notificationDecoder.decodePayload(responseBodyV2.getSignedPayload());
            log.info("appstore 接收回调信息decodedPayload值：{}", decodedPayload);
            String signedRenewalInfo = decodedPayload.getData().getSignedRenewalInfo();
            String signedTransactionInfo = decodedPayload.getData().getSignedTransactionInfo();

            JWSRenewalInfoDecodedPayload jwsRenewalInfoDecodedPayload = notificationDecoder.decodeRenewalInfo(signedRenewalInfo);
            JWSTransactionDecodedPayload jwsTransactionDecodedPayload = notificationDecoder.decodeTransaction(signedTransactionInfo);

            log.info("appstore 接收回调信息jwsRenewalInfoDecodedPayload值：{}", jwsRenewalInfoDecodedPayload);
            log.info("appstore 接收回调信息jwsTransactionDecodedPayload值：{}", jwsTransactionDecodedPayload);
            String lockKey = decodedPayload.getNotificationType().toString().concat(decodedPayload.getNotificationUUID()) + "-RedissonLock";
            // 获取 RLock 对象
            RLock rLock = redissonClient.getLock(lockKey);
            try {
                // 上锁
                boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                // 业务逻辑开始
                if (resultLock) {
                    log.info("上锁成功, 当前事件：{}",decodedPayload.getNotificationType());
                    switch (decodedPayload.getNotificationType()) {
                        // 退款
                        case REFUND:
//                                appStoreWebhookService.subscriptionCancel(jwsRenewalInfoDecodedPayload.getOriginalTransactionId());
                            break;
                        // 订阅已成功续订
                        case DID_RENEW:
                            if (decodedPayload.getData().getStatus().equals(1)){
                                appStoreWebhookService.subscriptionPaymentSucceeded(false,jwsRenewalInfoDecodedPayload.getOriginalTransactionId(),jwsTransactionDecodedPayload.getTransactionId());
                            }
                            break;
                        // 订阅已过期
                        case DID_CHANGE_RENEWAL_STATUS:
                            if (Subtype.AUTO_RENEW_DISABLED.equals(decodedPayload.getSubtype())){
                                appStoreWebhookService.subscriptionCancel(jwsRenewalInfoDecodedPayload.getOriginalTransactionId());
                            }
                            break;
                        default:
                            log.info(ErrorMessage.PAYMENT_APPSTORE_WEBHOOK_EVENT_NO_HANDLE.concat(decodedPayload.getNotificationType().toString()));
//                            throw new BackendRuntimeException(ErrorMessage.PAYMENT_APPSTORE_WEBHOOK_EVENT_NO_HANDLE.concat(decodedPayload.getNotificationType().toString()));
                    }
                }
            } catch (InterruptedException e) {
                log.info("上锁失败");
                log.error("上锁失败".concat(e.getMessage()),e);
            } finally {
                if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                    rLock.unlock();
                }
            }

        } catch (JsonProcessingException | CertificateException e) {
            log.error("appstore 回调失败：" + e.getMessage(), e);
            e.printStackTrace();
        }


//        //记录每次的苹果通知请求
//        //解析苹果请求的数据
//        String signedPayload=new String(Base64.decodeBase64(signedPayload1.split("\\.")[0]));
////        String result2=java.util.Base64.getUrlDecoder().decode(signedPayload).toString();
//        JSONObject jsonObject=JSONObject.parseObject(signedPayload);
//
//        Jws<Claims> result=jwsUtil.verifyJWT(jsonObject.getJSONArray("x5c").get(0).toString(),signedPayload);
//        System.out.println(result.toString());
//
//        String notificationType=result.getBody().get("notificationType").toString();
//        Claims map=result.getBody();
//        HashMap<String,Object> envmap=map.get("data",HashMap.class);
//        String env=envmap.get("environment").toString();
//        String resulttran=new String(Base64.decodeBase64(envmap.get("signedTransactionInfo").toString().split("\\.")[0]));
//        JSONObject jsonObjecttran=JSONObject.parseObject(resulttran);
//
//        Jws<Claims> result3=jwsUtil.verifyJWT(jsonObjecttran.getJSONArray("x5c").get(0).toString(),envmap.get("signedTransactionInfo").toString());
//        System.out.println(result3.getBody().toString());
//        log.info("applePay 回调：{}",result3.getBody());
//        log.info("苹果通知类型："+notificationType+"环境:"+env);
//
//
//        if(notificationType.equals("DID_RENEW")) {
//            String transactionId = result3.getBody().get("transactionId").toString();
//            String productId = result3.getBody().get("productId").toString();
//            String originalTransactionId = result3.getBody().get("originalTransactionId").toString();
//        }
//        else if(notificationType.equals("REFUND")) {
//
//            String originalTransactionId= result3.getBody().get("originalTransactionId").toString();
//            String productId = result3.getBody().get("productId").toString();
//            //逻辑代码
//        }
//        else {
//            log.info("notificationType未处理：" + notificationType);
//        }


        // 返回200 OK响应
        return ResponseEntity.ok().build();
    }


    public static void main(String[] args) {
        String url = "https://api.storekit-sandbox.itunes.apple.com/inApps/v1/notifications/test";

        // 设置请求头部
        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");
        headers.set("Authorization", "Bearer YOUR_ACCESS_TOKEN");

        // 设置请求体
        String requestBody = "{\"notification_type\": \"TEST\"}";

        // 发送 POST 请求
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<String> request = new HttpEntity<>(requestBody, headers);
        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, request, String.class);

        // 处理响应
        HttpStatus statusCode = response.getStatusCode();
        String responseBody = response.getBody();
        System.out.println("Status Code: " + statusCode);
        System.out.println("Response Body: " + responseBody);
    }
}
